# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import *

from ....api import Request, Response, set_timeout, set_tenant_key, set_user_access_token, set_path_params, \
    set_query_params, set_response_stream, set_is_response_stream, FormData, FormDataFile
from ....config import Config
from ....consts import ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_APP
from .model import *


class Service(object):
    def __init__(self, conf):
        # type: (Config) -> None
        self.conf = conf
        self.files = FileService(self)
        self.file_comments = FileCommentService(self)
        self.file_comment_replys = FileCommentReplyService(self)
        self.medias = MediaService(self)
        



class FileService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def upload_finish(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (FileUploadFinishReqBody, str, str, int) -> FileUploadFinishReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return FileUploadFinishReqCall(self, body, request_opts=request_opts)

    def upload_prepare(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (UploadInfo, str, str, int) -> FileUploadPrepareReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return FileUploadPrepareReqCall(self, body, request_opts=request_opts)

    def upload_all(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> FileUploadAllReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return FileUploadAllReqCall(self, request_opts=request_opts)

    def upload_part(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> FileUploadPartReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return FileUploadPartReqCall(self, request_opts=request_opts)

    def download(self, tenant_key=None, user_access_token=None, response_stream=None, timeout=None):
        # type: (str, str, Union[None, IO], int) -> FileDownloadReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        if response_stream is not None:
            request_opts += [set_response_stream(response_stream)]

        return FileDownloadReqCall(self, request_opts=request_opts)

    def subscribe(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> FileSubscribeReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return FileSubscribeReqCall(self, request_opts=request_opts)


class FileCommentService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def create(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (FileComment, str, str, int) -> FileCommentCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return FileCommentCreateReqCall(self, body, request_opts=request_opts)

    def get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> FileCommentGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return FileCommentGetReqCall(self, request_opts=request_opts)

    def list(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> FileCommentListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return FileCommentListReqCall(self, request_opts=request_opts)

    def patch(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (FileCommentPatchReqBody, str, str, int) -> FileCommentPatchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return FileCommentPatchReqCall(self, body, request_opts=request_opts)


class FileCommentReplyService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def update(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (FileCommentReplyUpdateReqBody, str, str, int) -> FileCommentReplyUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return FileCommentReplyUpdateReqCall(self, body, request_opts=request_opts)

    def delete(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> FileCommentReplyDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return FileCommentReplyDeleteReqCall(self, request_opts=request_opts)


class MediaService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def upload_part(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> MediaUploadPartReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return MediaUploadPartReqCall(self, request_opts=request_opts)

    def upload_all(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> MediaUploadAllReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return MediaUploadAllReqCall(self, request_opts=request_opts)

    def upload_finish(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (MediaUploadFinishReqBody, str, str, int) -> MediaUploadFinishReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return MediaUploadFinishReqCall(self, body, request_opts=request_opts)

    def upload_prepare(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (UploadInfo, str, str, int) -> MediaUploadPrepareReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return MediaUploadPrepareReqCall(self, body, request_opts=request_opts)

    def batch_get_tmp_download_url(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> MediaBatchGetTmpDownloadUrlReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return MediaBatchGetTmpDownloadUrlReqCall(self, request_opts=request_opts)

    def download(self, tenant_key=None, user_access_token=None, response_stream=None, timeout=None):
        # type: (str, str, Union[None, IO], int) -> MediaDownloadReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        if response_stream is not None:
            request_opts += [set_response_stream(response_stream)]

        return MediaDownloadReqCall(self, request_opts=request_opts)



class MediaUploadPartReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MediaService, List[Any]) -> None

        self.service = service
        self.body = FormData()

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_upload_id(self, upload_id):
        # type: (str) -> MediaUploadPartReqCall
        self.body.add_param('upload_id', upload_id)
        return self

    def set_seq(self, seq):
        # type: (int) -> MediaUploadPartReqCall
        self.body.add_param('seq', seq)
        return self

    def set_size(self, size):
        # type: (int) -> MediaUploadPartReqCall
        self.body.add_param('size', size)
        return self

    def set_checksum(self, checksum):
        # type: (str) -> MediaUploadPartReqCall
        self.body.add_param('checksum', checksum)
        return self

    def set_file(self, file):
        # type: (IO[Any]) -> MediaUploadPartReqCall
        self.body.add_file('file', FormDataFile(file))
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('drive/v1/medias/upload_part', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT], self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileUploadFinishReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (FileService, FileUploadFinishReqBody, List[Any]) -> None

        self.service = service
        self.body = body

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[FileUploadFinishResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('drive/v1/files/upload_finish', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=FileUploadFinishResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileUploadPrepareReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (FileService, UploadInfo, List[Any]) -> None

        self.service = service
        self.body = body

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[FileUploadPrepareResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('drive/v1/files/upload_prepare', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=FileUploadPrepareResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MediaUploadAllReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MediaService, List[Any]) -> None

        self.service = service
        self.body = FormData()

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_name(self, file_name):
        # type: (str) -> MediaUploadAllReqCall
        self.body.add_param('file_name', file_name)
        return self

    def set_parent_type(self, parent_type):
        # type: (str) -> MediaUploadAllReqCall
        self.body.add_param('parent_type', parent_type)
        return self

    def set_parent_node(self, parent_node):
        # type: (str) -> MediaUploadAllReqCall
        self.body.add_param('parent_node', parent_node)
        return self

    def set_size(self, size):
        # type: (int) -> MediaUploadAllReqCall
        self.body.add_param('size', size)
        return self

    def set_checksum(self, checksum):
        # type: (str) -> MediaUploadAllReqCall
        self.body.add_param('checksum', checksum)
        return self

    def set_file(self, file):
        # type: (IO[Any]) -> MediaUploadAllReqCall
        self.body.add_file('file', FormDataFile(file))
        return self

    def do(self):
        # type: () -> Response[MediaUploadAllResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('drive/v1/medias/upload_all', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT], self.body, output_class=MediaUploadAllResult , request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MediaUploadFinishReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (MediaService, MediaUploadFinishReqBody, List[Any]) -> None

        self.service = service
        self.body = body

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[MediaUploadFinishResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('drive/v1/medias/upload_finish', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=MediaUploadFinishResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileUploadAllReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (FileService, List[Any]) -> None

        self.service = service
        self.body = FormData()

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_name(self, file_name):
        # type: (str) -> FileUploadAllReqCall
        self.body.add_param('file_name', file_name)
        return self

    def set_parent_type(self, parent_type):
        # type: (str) -> FileUploadAllReqCall
        self.body.add_param('parent_type', parent_type)
        return self

    def set_parent_node(self, parent_node):
        # type: (str) -> FileUploadAllReqCall
        self.body.add_param('parent_node', parent_node)
        return self

    def set_size(self, size):
        # type: (int) -> FileUploadAllReqCall
        self.body.add_param('size', size)
        return self

    def set_checksum(self, checksum):
        # type: (str) -> FileUploadAllReqCall
        self.body.add_param('checksum', checksum)
        return self

    def set_file(self, file):
        # type: (IO[Any]) -> FileUploadAllReqCall
        self.body.add_file('file', FormDataFile(file))
        return self

    def do(self):
        # type: () -> Response[FileUploadAllResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('drive/v1/files/upload_all', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT], self.body, output_class=FileUploadAllResult , request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MediaUploadPrepareReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (MediaService, UploadInfo, List[Any]) -> None

        self.service = service
        self.body = body

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[MediaUploadPrepareResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('drive/v1/medias/upload_prepare', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=MediaUploadPrepareResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileUploadPartReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (FileService, List[Any]) -> None

        self.service = service
        self.body = FormData()

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_upload_id(self, upload_id):
        # type: (str) -> FileUploadPartReqCall
        self.body.add_param('upload_id', upload_id)
        return self

    def set_seq(self, seq):
        # type: (int) -> FileUploadPartReqCall
        self.body.add_param('seq', seq)
        return self

    def set_size(self, size):
        # type: (int) -> FileUploadPartReqCall
        self.body.add_param('size', size)
        return self

    def set_checksum(self, checksum):
        # type: (str) -> FileUploadPartReqCall
        self.body.add_param('checksum', checksum)
        return self

    def set_file(self, file):
        # type: (IO[Any]) -> FileUploadPartReqCall
        self.body.add_file('file', FormDataFile(file))
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('drive/v1/files/upload_part', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT], self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MediaBatchGetTmpDownloadUrlReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MediaService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_tokens(self, file_tokens):
        # type: (List[str]) -> MediaBatchGetTmpDownloadUrlReqCall
        self.query_params['file_tokens'] = file_tokens
        return self

    def do(self):
        # type: () -> Response[MediaBatchGetTmpDownloadUrlResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('drive/v1/medias/batch_get_tmp_download_url', 'GET', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=MediaBatchGetTmpDownloadUrlResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileDownloadReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (FileService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_token(self, file_token):
        # type: (str) -> FileDownloadReqCall
        self.path_params['file_token'] = file_token
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_is_response_stream()]
        req = Request('drive/v1/files/:file_token/download', 'GET', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MediaDownloadReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MediaService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_token(self, file_token):
        # type: (str) -> MediaDownloadReqCall
        self.path_params['file_token'] = file_token
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_is_response_stream()]
        req = Request('drive/v1/medias/:file_token/download', 'GET', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileSubscribeReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (FileService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_token(self, file_token):
        # type: (str) -> FileSubscribeReqCall
        self.path_params['file_token'] = file_token
        return self

    def set_file_type(self, file_type):
        # type: (str) -> FileSubscribeReqCall
        self.query_params['file_type'] = file_type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('drive/v1/files/:file_token/subscribe', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileCommentCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (FileCommentService, FileComment, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_token(self, file_token):
        # type: (str) -> FileCommentCreateReqCall
        self.path_params['file_token'] = file_token
        return self

    def set_file_type(self, file_type):
        # type: (str) -> FileCommentCreateReqCall
        self.query_params['file_type'] = file_type
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> FileCommentCreateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[FileComment]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('drive/v1/files/:file_token/comments', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=FileComment, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileCommentGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (FileCommentService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_token(self, file_token):
        # type: (str) -> FileCommentGetReqCall
        self.path_params['file_token'] = file_token
        return self

    def set_comment_id(self, comment_id):
        # type: (int) -> FileCommentGetReqCall
        self.path_params['comment_id'] = comment_id
        return self

    def set_file_type(self, file_type):
        # type: (str) -> FileCommentGetReqCall
        self.query_params['file_type'] = file_type
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> FileCommentGetReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[FileComment]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('drive/v1/files/:file_token/comments/:comment_id', 'GET', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=FileComment, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileCommentReplyUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (FileCommentReplyService, FileCommentReplyUpdateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_token(self, file_token):
        # type: (str) -> FileCommentReplyUpdateReqCall
        self.path_params['file_token'] = file_token
        return self

    def set_comment_id(self, comment_id):
        # type: (int) -> FileCommentReplyUpdateReqCall
        self.path_params['comment_id'] = comment_id
        return self

    def set_reply_id(self, reply_id):
        # type: (int) -> FileCommentReplyUpdateReqCall
        self.path_params['reply_id'] = reply_id
        return self

    def set_file_type(self, file_type):
        # type: (str) -> FileCommentReplyUpdateReqCall
        self.query_params['file_type'] = file_type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('drive/v1/files/:file_token/comments/:comment_id/replies/:reply_id', 'PUT', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileCommentListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (FileCommentService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_token(self, file_token):
        # type: (str) -> FileCommentListReqCall
        self.path_params['file_token'] = file_token
        return self

    def set_file_type(self, file_type):
        # type: (str) -> FileCommentListReqCall
        self.query_params['file_type'] = file_type
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> FileCommentListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_is_solved(self, is_solved):
        # type: (bool) -> FileCommentListReqCall
        self.query_params['is_solved'] = is_solved
        return self

    def set_page_token(self, page_token):
        # type: (str) -> FileCommentListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> FileCommentListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[FileCommentListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('drive/v1/files/:file_token/comments', 'GET', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=FileCommentListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileCommentPatchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (FileCommentService, FileCommentPatchReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_token(self, file_token):
        # type: (str) -> FileCommentPatchReqCall
        self.path_params['file_token'] = file_token
        return self

    def set_comment_id(self, comment_id):
        # type: (int) -> FileCommentPatchReqCall
        self.path_params['comment_id'] = comment_id
        return self

    def set_file_type(self, file_type):
        # type: (str) -> FileCommentPatchReqCall
        self.query_params['file_type'] = file_type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('drive/v1/files/:file_token/comments/:comment_id', 'PATCH', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileCommentReplyDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (FileCommentReplyService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_token(self, file_token):
        # type: (str) -> FileCommentReplyDeleteReqCall
        self.path_params['file_token'] = file_token
        return self

    def set_comment_id(self, comment_id):
        # type: (int) -> FileCommentReplyDeleteReqCall
        self.path_params['comment_id'] = comment_id
        return self

    def set_reply_id(self, reply_id):
        # type: (int) -> FileCommentReplyDeleteReqCall
        self.path_params['reply_id'] = reply_id
        return self

    def set_file_type(self, file_type):
        # type: (str) -> FileCommentReplyDeleteReqCall
        self.query_params['file_type'] = file_type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('drive/v1/files/:file_token/comments/:comment_id/replies/:reply_id', 'DELETE', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp




# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
from ....event.model.event import *
import attr




@to_json_decorator
@attr.s
class I18nContent(object):
    locale = attr.ib(type=str, default=None, metadata={'json': 'locale'})
    value = attr.ib(type=str, default=None, metadata={'json': 'value'})


@to_json_decorator
@attr.s
class EmployeeTypeEnum(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["enum_value"])
    enum_id = attr.ib(type=str, default=None, metadata={'json': 'enum_id'})
    enum_value = attr.ib(type=int, default=None, metadata={'json': 'enum_value'})
    content = attr.ib(type=str, default=None, metadata={'json': 'content'})
    enum_type = attr.ib(type=int, default=None, metadata={'json': 'enum_type'})
    enum_status = attr.ib(type=int, default=None, metadata={'json': 'enum_status'})
    i18n_content = attr.ib(type=List[I18nContent], default=None, metadata={'json': 'i18n_content'})


@to_json_decorator
@attr.s
class NotificationOption(object):
    channels = attr.ib(type=List[str], default=None, metadata={'json': 'channels'})
    language = attr.ib(type=str, default=None, metadata={'json': 'language'})


@to_json_decorator
@attr.s
class UserGroup(object):
    user_group_id = attr.ib(type=str, default=None, metadata={'json': 'user_group_id'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    type = attr.ib(type=int, default=None, metadata={'json': 'type'})
    member_count = attr.ib(type=int, default=None, metadata={'json': 'member_count'})
    status = attr.ib(type=int, default=None, metadata={'json': 'status'})


@to_json_decorator
@attr.s
class DepartmentStatus(object):
    is_deleted = attr.ib(type=bool, default=None, metadata={'json': 'is_deleted'})


@to_json_decorator
@attr.s
class OldDepartmentObject(object):
    status = attr.ib(type=DepartmentStatus, default=None, metadata={'json': 'status'})


@to_json_decorator
@attr.s
class DepartmentEvent(object):
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    parent_department_id = attr.ib(type=str, default=None, metadata={'json': 'parent_department_id'})
    department_id = attr.ib(type=str, default=None, metadata={'json': 'department_id'})
    open_department_id = attr.ib(type=str, default=None, metadata={'json': 'open_department_id'})
    leader_user_id = attr.ib(type=str, default=None, metadata={'json': 'leader_user_id'})
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    order = attr.ib(type=int, default=None, metadata={'json': 'order'})
    
    status = attr.ib(type=DepartmentStatus, default=None, metadata={'json': 'status'})


@to_json_decorator
@attr.s
class DepartmentI18nName(object):
    zh_cn = attr.ib(type=str, default=None, metadata={'json': 'zh_cn'})
    ja_jp = attr.ib(type=str, default=None, metadata={'json': 'ja_jp'})
    en_us = attr.ib(type=str, default=None, metadata={'json': 'en_us'})


@to_json_decorator
@attr.s
class Department(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["order"])
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    i18n_name = attr.ib(type=DepartmentI18nName, default=None, metadata={'json': 'i18n_name'})
    parent_department_id = attr.ib(type=str, default=None, metadata={'json': 'parent_department_id'})
    department_id = attr.ib(type=str, default=None, metadata={'json': 'department_id'})
    open_department_id = attr.ib(type=str, default=None, metadata={'json': 'open_department_id'})
    leader_user_id = attr.ib(type=str, default=None, metadata={'json': 'leader_user_id'})
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    order = attr.ib(type=int, default=None, metadata={'json': 'order'})
    unit_ids = attr.ib(type=List[str], default=None, metadata={'json': 'unit_ids'})
    member_count = attr.ib(type=int, default=None, metadata={'json': 'member_count'})
    status = attr.ib(type=DepartmentStatus, default=None, metadata={'json': 'status'})
    create_group_chat = attr.ib(type=bool, default=None, metadata={'json': 'create_group_chat'})


@to_json_decorator
@attr.s
class UserPosition(object):
    position_code = attr.ib(type=str, default=None, metadata={'json': 'position_code'})
    position_name = attr.ib(type=str, default=None, metadata={'json': 'position_name'})
    department_id = attr.ib(type=str, default=None, metadata={'json': 'department_id'})
    leader_user_id = attr.ib(type=str, default=None, metadata={'json': 'leader_user_id'})
    leader_position_code = attr.ib(type=str, default=None, metadata={'json': 'leader_position_code'})
    is_major = attr.ib(type=bool, default=None, metadata={'json': 'is_major'})


@to_json_decorator
@attr.s
class AvatarInfo(object):
    avatar_72 = attr.ib(type=str, default=None, metadata={'json': 'avatar_72'})
    avatar_240 = attr.ib(type=str, default=None, metadata={'json': 'avatar_240'})
    avatar_640 = attr.ib(type=str, default=None, metadata={'json': 'avatar_640'})
    avatar_origin = attr.ib(type=str, default=None, metadata={'json': 'avatar_origin'})


@to_json_decorator
@attr.s
class UserStatus(object):
    is_frozen = attr.ib(type=bool, default=None, metadata={'json': 'is_frozen'})
    is_resigned = attr.ib(type=bool, default=None, metadata={'json': 'is_resigned'})
    is_activated = attr.ib(type=bool, default=None, metadata={'json': 'is_activated'})


@to_json_decorator
@attr.s
class UserOrder(object):
    department_id = attr.ib(type=str, default=None, metadata={'json': 'department_id'})
    user_order = attr.ib(type=int, default=None, metadata={'json': 'user_order'})
    department_order = attr.ib(type=int, default=None, metadata={'json': 'department_order'})


@to_json_decorator
@attr.s
class UserCustomAttrValue(object):
    text = attr.ib(type=str, default=None, metadata={'json': 'text'})
    url = attr.ib(type=str, default=None, metadata={'json': 'url'})
    pc_url = attr.ib(type=str, default=None, metadata={'json': 'pc_url'})


@to_json_decorator
@attr.s
class UserCustomAttr(object):
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    id = attr.ib(type=str, default=None, metadata={'json': 'id'})
    value = attr.ib(type=UserCustomAttrValue, default=None, metadata={'json': 'value'})


@to_json_decorator
@attr.s
class UserEvent(object):
    open_id = attr.ib(type=str, default=None, metadata={'json': 'open_id'})
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    en_name = attr.ib(type=str, default=None, metadata={'json': 'en_name'})
    email = attr.ib(type=str, default=None, metadata={'json': 'email'})
    mobile = attr.ib(type=str, default=None, metadata={'json': 'mobile'})
    
    gender = attr.ib(type=int, default=None, metadata={'json': 'gender'})
    avatar = attr.ib(type=AvatarInfo, default=None, metadata={'json': 'avatar'})
    status = attr.ib(type=UserStatus, default=None, metadata={'json': 'status'})
    department_ids = attr.ib(type=List[str], default=None, metadata={'json': 'department_ids'})
    leader_user_id = attr.ib(type=str, default=None, metadata={'json': 'leader_user_id'})
    city = attr.ib(type=str, default=None, metadata={'json': 'city'})
    country = attr.ib(type=str, default=None, metadata={'json': 'country'})
    work_station = attr.ib(type=str, default=None, metadata={'json': 'work_station'})
    join_time = attr.ib(type=int, default=None, metadata={'json': 'join_time'})
    
    employee_no = attr.ib(type=str, default=None, metadata={'json': 'employee_no'})
    employee_type = attr.ib(type=int, default=None, metadata={'json': 'employee_type'})
    
    orders = attr.ib(type=List[UserOrder], default=None, metadata={'json': 'orders'})
    
    custom_attrs = attr.ib(type=List[UserCustomAttr], default=None, metadata={'json': 'custom_attrs'})


@to_json_decorator
@attr.s
class User(object):
    union_id = attr.ib(type=str, default=None, metadata={'json': 'union_id'})
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    open_id = attr.ib(type=str, default=None, metadata={'json': 'open_id'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    en_name = attr.ib(type=str, default=None, metadata={'json': 'en_name'})
    email = attr.ib(type=str, default=None, metadata={'json': 'email'})
    mobile = attr.ib(type=str, default=None, metadata={'json': 'mobile'})
    mobile_visible = attr.ib(type=bool, default=None, metadata={'json': 'mobile_visible'})
    gender = attr.ib(type=int, default=None, metadata={'json': 'gender'})
    avatar_key = attr.ib(type=str, default=None, metadata={'json': 'avatar_key'})
    avatar = attr.ib(type=AvatarInfo, default=None, metadata={'json': 'avatar'})
    status = attr.ib(type=UserStatus, default=None, metadata={'json': 'status'})
    department_ids = attr.ib(type=List[str], default=None, metadata={'json': 'department_ids'})
    leader_user_id = attr.ib(type=str, default=None, metadata={'json': 'leader_user_id'})
    city = attr.ib(type=str, default=None, metadata={'json': 'city'})
    country = attr.ib(type=str, default=None, metadata={'json': 'country'})
    work_station = attr.ib(type=str, default=None, metadata={'json': 'work_station'})
    join_time = attr.ib(type=int, default=None, metadata={'json': 'join_time'})
    is_tenant_manager = attr.ib(type=bool, default=None, metadata={'json': 'is_tenant_manager'})
    employee_no = attr.ib(type=str, default=None, metadata={'json': 'employee_no'})
    employee_type = attr.ib(type=int, default=None, metadata={'json': 'employee_type'})
    
    orders = attr.ib(type=List[UserOrder], default=None, metadata={'json': 'orders'})
    custom_attrs = attr.ib(type=List[UserCustomAttr], default=None, metadata={'json': 'custom_attrs'})
    enterprise_email = attr.ib(type=str, default=None, metadata={'json': 'enterprise_email'})
    
    
    
    need_send_notification = attr.ib(type=bool, default=None, metadata={'json': 'need_send_notification'})
    notification_option = attr.ib(type=NotificationOption, default=None, metadata={'json': 'notification_option'})


@to_json_decorator
@attr.s
class Scope(object):
    departments = attr.ib(type=List[Department], default=None, metadata={'json': 'departments'})
    users = attr.ib(type=List[User], default=None, metadata={'json': 'users'})
    user_groups = attr.ib(type=List[UserGroup], default=None, metadata={'json': 'user_groups'})


@to_json_decorator
@attr.s
class CustomAttrEvent(object):
    contact_field_key = attr.ib(type=str, default=None, metadata={'json': 'contact_field_key'})
    field_value_types = attr.ib(type=str, default=None, metadata={'json': 'field_value_types'})
    i18n_field_names = attr.ib(type=str, default=None, metadata={'json': 'i18n_field_names'})
    allow_open_query = attr.ib(type=str, default=None, metadata={'json': 'allow_open_query'})


@to_json_decorator
@attr.s
class DepartmentUnit(object):
    unit_id = attr.ib(type=str, default=None, metadata={'json': 'unit_id'})
    unit_type = attr.ib(type=str, default=None, metadata={'json': 'unit_type'})
    unit_name = attr.ib(type=str, default=None, metadata={'json': 'unit_name'})


@to_json_decorator
@attr.s
class GroupEvent(object):
    user_group_id = attr.ib(type=str, default=None, metadata={'json': 'user_group_id'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})


@to_json_decorator
@attr.s
class UserGroupMember(object):
    pass


@to_json_decorator
@attr.s
class OldUserObject(object):
    department_ids = attr.ib(type=List[str], default=None, metadata={'json': 'department_ids'})




@attr.s
class DepartmentGetResult(object):
    department = attr.ib(type=Department, default=None, metadata={'json': 'department'})


@to_json_decorator
@attr.s
class UserDeleteReqBody(object):
    department_chat_acceptor_user_id = attr.ib(type=str, default=None, metadata={'json': 'department_chat_acceptor_user_id'})
    external_chat_acceptor_user_id = attr.ib(type=str, default=None, metadata={'json': 'external_chat_acceptor_user_id'})
    docs_acceptor_user_id = attr.ib(type=str, default=None, metadata={'json': 'docs_acceptor_user_id'})
    calendar_acceptor_user_id = attr.ib(type=str, default=None, metadata={'json': 'calendar_acceptor_user_id'})
    application_acceptor_user_id = attr.ib(type=str, default=None, metadata={'json': 'application_acceptor_user_id'})
    helpdesk_acceptor_user_id = attr.ib(type=str, default=None, metadata={'json': 'helpdesk_acceptor_user_id'})




@attr.s
class DepartmentListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    items = attr.ib(type=List[Department], default=None, metadata={'json': 'items'})


@to_json_decorator
@attr.s
class UserGroupUpdateUserGroupIdReqBody(object):
    new_user_group_id = attr.ib(type=str, default=None, metadata={'json': 'new_user_group_id'})






@attr.s
class DepartmentPatchResult(object):
    department = attr.ib(type=Department, default=None, metadata={'json': 'department'})



@attr.s
class UserUpdateResult(object):
    user = attr.ib(type=User, default=None, metadata={'json': 'user'})



@attr.s
class UserCreateResult(object):
    user = attr.ib(type=User, default=None, metadata={'json': 'user'})



@attr.s
class UserPatchResult(object):
    user = attr.ib(type=User, default=None, metadata={'json': 'user'})


@to_json_decorator
@attr.s
class UserUpdateUserIdReqBody(object):
    new_user_id = attr.ib(type=str, default=None, metadata={'json': 'new_user_id'})



@to_json_decorator
@attr.s
class UserGroupMemberCreateReqBody(object):
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})






@attr.s
class DepartmentCreateResult(object):
    department = attr.ib(type=Department, default=None, metadata={'json': 'department'})



@attr.s
class UserGroupPatchResult(object):
    user_group = attr.ib(type=UserGroup, default=None, metadata={'json': 'user_group'})


@to_json_decorator
@attr.s
class DepartmentUnitPatchReqBody(object):
    unit_type = attr.ib(type=str, default=None, metadata={'json': 'unit_type'})
    unit_name = attr.ib(type=str, default=None, metadata={'json': 'unit_name'})


@attr.s
class DepartmentUnitPatchResult(object):
    department_unit = attr.ib(type=DepartmentUnit, default=None, metadata={'json': 'department_unit'})



@attr.s
class UserGroupCreateResult(object):
    user_group = attr.ib(type=UserGroup, default=None, metadata={'json': 'user_group'})



@attr.s
class UserGroupListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    items = attr.ib(type=List[UserGroup], default=None, metadata={'json': 'items'})





@attr.s
class UserGetResult(object):
    user = attr.ib(type=User, default=None, metadata={'json': 'user'})



@attr.s
class UserGroupMemberListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    items = attr.ib(type=List[User], default=None, metadata={'json': 'items'})





@attr.s
class DepartmentUpdateResult(object):
    department = attr.ib(type=Department, default=None, metadata={'json': 'department'})



@attr.s
class DepartmentUnitCreateResult(object):
    department_unit = attr.ib(type=DepartmentUnit, default=None, metadata={'json': 'department_unit'})



@attr.s
class UserListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    items = attr.ib(type=List[User], default=None, metadata={'json': 'items'})



@attr.s
class UserGroupGetResult(object):
    user_group = attr.ib(type=UserGroup, default=None, metadata={'json': 'user_group'})



@attr.s
class DepartmentParentResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    items = attr.ib(type=List[Department], default=None, metadata={'json': 'items'})


@to_json_decorator
@attr.s
class DepartmentUpdateDepartmentIdReqBody(object):
    new_department_id = attr.ib(type=str, default=None, metadata={'json': 'new_department_id'})



@to_json_decorator
@attr.s
class DepartmentUnbindDepartmentChatReqBody(object):
    department_id = attr.ib(type=str, default=None, metadata={'json': 'department_id'})



@to_json_decorator
@attr.s
class DepartmentSearchReqBody(object):
    query = attr.ib(type=str, default=None, metadata={'json': 'query'})


@attr.s
class DepartmentSearchResult(object):
    items = attr.ib(type=List[Department], default=None, metadata={'json': 'items'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})



@attr.s
class EmployeeTypeEnumListResult(object):
    items = attr.ib(type=List[EmployeeTypeEnum], default=None, metadata={'json': 'items'})
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})


@attr.s
class DepartmentCreatedEventData(object):
    object = attr.ib(type=DepartmentEvent, default=None, metadata={'json': 'object'})


@attr.s
class DepartmentCreatedEvent(BaseEventV2):
    event = attr.ib(type=DepartmentCreatedEventData, default=None)



@attr.s
class DepartmentDeletedEventData(object):
    object = attr.ib(type=DepartmentEvent, default=None, metadata={'json': 'object'})
    old_object = attr.ib(type=OldDepartmentObject, default=None, metadata={'json': 'old_object'})


@attr.s
class DepartmentDeletedEvent(BaseEventV2):
    event = attr.ib(type=DepartmentDeletedEventData, default=None)



@attr.s
class DepartmentUpdatedEventData(object):
    object = attr.ib(type=DepartmentEvent, default=None, metadata={'json': 'object'})
    old_object = attr.ib(type=DepartmentEvent, default=None, metadata={'json': 'old_object'})


@attr.s
class DepartmentUpdatedEvent(BaseEventV2):
    event = attr.ib(type=DepartmentUpdatedEventData, default=None)



@attr.s
class UserCreatedEventData(object):
    object = attr.ib(type=UserEvent, default=None, metadata={'json': 'object'})


@attr.s
class UserCreatedEvent(BaseEventV2):
    event = attr.ib(type=UserCreatedEventData, default=None)



@attr.s
class UserDeletedEventData(object):
    object = attr.ib(type=UserEvent, default=None, metadata={'json': 'object'})
    old_object = attr.ib(type=OldUserObject, default=None, metadata={'json': 'old_object'})


@attr.s
class UserDeletedEvent(BaseEventV2):
    event = attr.ib(type=UserDeletedEventData, default=None)



@attr.s
class UserUpdatedEventData(object):
    object = attr.ib(type=UserEvent, default=None, metadata={'json': 'object'})
    old_object = attr.ib(type=UserEvent, default=None, metadata={'json': 'old_object'})


@attr.s
class UserUpdatedEvent(BaseEventV2):
    event = attr.ib(type=UserUpdatedEventData, default=None)



@attr.s
class UserGroupCreatedEventData(object):
    object = attr.ib(type=GroupEvent, default=None, metadata={'json': 'object'})


@attr.s
class UserGroupCreatedEvent(BaseEventV2):
    event = attr.ib(type=UserGroupCreatedEventData, default=None)



@attr.s
class UserGroupDeletedEventData(object):
    object = attr.ib(type=GroupEvent, default=None, metadata={'json': 'object'})


@attr.s
class UserGroupDeletedEvent(BaseEventV2):
    event = attr.ib(type=UserGroupDeletedEventData, default=None)



@attr.s
class UserGroupUpdatedEventData(object):
    object = attr.ib(type=GroupEvent, default=None, metadata={'json': 'object'})
    old_object = attr.ib(type=GroupEvent, default=None, metadata={'json': 'old_object'})


@attr.s
class UserGroupUpdatedEvent(BaseEventV2):
    event = attr.ib(type=UserGroupUpdatedEventData, default=None)



@attr.s
class ScopeUpdatedEventData(object):
    added = attr.ib(type=Scope, default=None, metadata={'json': 'added'})
    removed = attr.ib(type=Scope, default=None, metadata={'json': 'removed'})


@attr.s
class ScopeUpdatedEvent(BaseEventV2):
    event = attr.ib(type=ScopeUpdatedEventData, default=None)



@attr.s
class UserGroupMemberChangedEventData(object):
    object = attr.ib(type=GroupEvent, default=None, metadata={'json': 'object'})


@attr.s
class UserGroupMemberChangedEvent(BaseEventV2):
    event = attr.ib(type=UserGroupMemberChangedEventData, default=None)



@attr.s
class CustomAttrEventUpdatedEventData(object):
    object = attr.ib(type=CustomAttrEvent, default=None, metadata={'json': 'object'})
    old_object = attr.ib(type=CustomAttrEvent, default=None, metadata={'json': 'old_object'})


@attr.s
class CustomAttrEventUpdatedEvent(BaseEventV2):
    event = attr.ib(type=CustomAttrEventUpdatedEventData, default=None)



@attr.s
class EmployeeTypeEnumActivedEventData(object):
    old_enum = attr.ib(type=EmployeeTypeEnum, default=None, metadata={'json': 'old_enum'})
    new_enum = attr.ib(type=EmployeeTypeEnum, default=None, metadata={'json': 'new_enum'})


@attr.s
class EmployeeTypeEnumActivedEvent(BaseEventV2):
    event = attr.ib(type=EmployeeTypeEnumActivedEventData, default=None)



@attr.s
class EmployeeTypeEnumCreatedEventData(object):
    new_enum = attr.ib(type=EmployeeTypeEnum, default=None, metadata={'json': 'new_enum'})


@attr.s
class EmployeeTypeEnumCreatedEvent(BaseEventV2):
    event = attr.ib(type=EmployeeTypeEnumCreatedEventData, default=None)



@attr.s
class EmployeeTypeEnumDeactivatedEventData(object):
    old_enum = attr.ib(type=EmployeeTypeEnum, default=None, metadata={'json': 'old_enum'})
    new_enum = attr.ib(type=EmployeeTypeEnum, default=None, metadata={'json': 'new_enum'})


@attr.s
class EmployeeTypeEnumDeactivatedEvent(BaseEventV2):
    event = attr.ib(type=EmployeeTypeEnumDeactivatedEventData, default=None)



@attr.s
class EmployeeTypeEnumDeletedEventData(object):
    old_enum = attr.ib(type=EmployeeTypeEnum, default=None, metadata={'json': 'old_enum'})


@attr.s
class EmployeeTypeEnumDeletedEvent(BaseEventV2):
    event = attr.ib(type=EmployeeTypeEnumDeletedEventData, default=None)



@attr.s
class EmployeeTypeEnumUpdatedEventData(object):
    old_enum = attr.ib(type=EmployeeTypeEnum, default=None, metadata={'json': 'old_enum'})
    new_enum = attr.ib(type=EmployeeTypeEnum, default=None, metadata={'json': 'new_enum'})


@attr.s
class EmployeeTypeEnumUpdatedEvent(BaseEventV2):
    event = attr.ib(type=EmployeeTypeEnumUpdatedEventData, default=None)



# -*- coding: UTF-8 -*-

import json
from typing import Union

from ...config import Config
from ...consts import *
from ...context import Context
from ...model import OapiResponse
from ...utils import make_datatype, decrypt
from ..handler.manager import Manager
from ..model.error import HandlerNotFoundError
from ..model.event import Fuzzy, HTTPEvent

VERSION_1 = "1.0"
VERSION_2 = "2.0"
RESPONSE_FORMAT = '{"codemsg":"%s"}'
CHALLENGE_RESPONSE_FORMAT = '{"challenge":"%s"}'
CALLBACK_TYPE_EVENT = "event_callback"
CALLBACK_TYPE_CHALLENGE = "url_verification"

HTTP_STATUS_OK = 200
HTTP_STATUS_INTERNAL_SERVER_ERROR = 500


class EventHandlers(object):

    def __init__(self, conf, httpEvent):
        # type: (Config, HTTPEvent) -> None
        self.httpEvent = httpEvent
        self.conf = conf
        self.ctx = Context()

    def parse(self):  # type: () -> None
        httpEvent, conf, ctx = self.httpEvent, self.conf, self.ctx
        req = httpEvent.request
        ctx.set(HTTP_HEADER_KEY, req.header)
        encrypt_key = conf.app_settings.encrypt_key
        if encrypt_key is not None and encrypt_key != "":
            req.body = decrypt(req.body, conf.app_settings.encrypt_key.encode("utf-8"))
        eventDict = json.loads(req.body)

        fuzzy = make_datatype(Fuzzy, eventDict)
        schema = VERSION_1
        if fuzzy.schema:
            schema = fuzzy.schema
        token = fuzzy.token
        event_type = CALLBACK_TYPE_CHALLENGE
        if fuzzy.event:
            event_type = fuzzy.event.type
        if fuzzy.header:
            token, event_type = fuzzy.header.token, fuzzy.header.event_type
        httpEvent.schema = schema
        httpEvent.event_type = event_type
        httpEvent.type = fuzzy.type
        httpEvent.challenge = fuzzy.challenge

        if token != conf.app_settings.verification_token:
            raise RuntimeError('New token invalid, received=%s, configured=%s' % (
                token, conf.app_settings.verification_token))

        data_type = Manager.get_event_type(conf, httpEvent.event_type)
        httpEvent.data = make_datatype(data_type, eventDict)

    def handler(self):  # type: () -> None
        ctx, conf, event = self.ctx, self.conf, self.httpEvent
        if event.type == CALLBACK_TYPE_CHALLENGE:
            return

        h = Manager.get_event_callback(conf, event.event_type)
        if h is None:
            raise HandlerNotFoundError(
                'event-type:%s, handler not found' % event.event_type)

        event.ctx = ctx
        h(ctx, conf, event.data)

    def complement(self, exception):  # type: (Union[None, Exception]) -> None
        conf, event = self.conf, self.httpEvent
        if exception:
            if isinstance(exception, HandlerNotFoundError):
                body = RESPONSE_FORMAT % exception.__str__()
                self.write_response(
                    HTTP_STATUS_OK, DEFAULT_CONTENT_TYPE, body.encode('utf-8'))
            else:
                err_msg = 'err occurred, %s' % exception.__str__()
                body = RESPONSE_FORMAT % exception.__str__()
                self.write_response(HTTP_STATUS_INTERNAL_SERVER_ERROR, DEFAULT_CONTENT_TYPE,
                                    body.encode('utf-8'))
                conf.logger.error(err_msg)
            return

        if event.type == CALLBACK_TYPE_CHALLENGE:
            self.write_response(HTTP_STATUS_OK, DEFAULT_CONTENT_TYPE,
                                (CHALLENGE_RESPONSE_FORMAT % event.challenge).encode('utf-8'))
            return

        self.write_response(HTTP_STATUS_OK, DEFAULT_CONTENT_TYPE,
                            (RESPONSE_FORMAT % 'success').encode('utf-8'))

    def write_response(self, status_code, content_type, body):
        # type: (int, str, Union[str, bytes]) -> None
        resp = OapiResponse()

        if isinstance(body, str):
            body = body.encode('utf-8')
        resp.status_code = status_code
        resp.content_type = content_type
        resp.body = body
        self.httpEvent.response = resp

    def handle(self):  # type: () -> None
        exception = None  # type: Union[None, Exception]
        try:
            self.parse()
            self.handler()
        except Exception as e:
            exception = e
        finally:
            self.complement(exception)



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import *

from ....api import Request, Response, set_timeout, set_tenant_key, set_user_access_token, set_path_params, \
    set_query_params, set_response_stream, set_is_response_stream, FormData, FormDataFile
from ....config import Config
from ....consts import ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_APP
from .model import *


class Service(object):
    def __init__(self, conf):
        # type: (Config) -> None
        self.conf = conf
        self.spreadsheetss = SpreadsheetsService(self)
        



class SpreadsheetsService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def condition_formats_batch_create(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsConditionFormatsBatchCreateReqBody, str, str, int) -> SpreadsheetsConditionFormatsBatchCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsConditionFormatsBatchCreateReqCall(self, body, request_opts=request_opts)

    def condition_formats_batch_delete(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsConditionFormatsBatchDeleteReqBody, str, str, int) -> SpreadsheetsConditionFormatsBatchDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsConditionFormatsBatchDeleteReqCall(self, body, request_opts=request_opts)

    def condition_formats_batch_get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> SpreadsheetsConditionFormatsBatchGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsConditionFormatsBatchGetReqCall(self, request_opts=request_opts)

    def condition_formats_batch_update(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsConditionFormatsBatchUpdateReqBody, str, str, int) -> SpreadsheetsConditionFormatsBatchUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsConditionFormatsBatchUpdateReqCall(self, body, request_opts=request_opts)

    def data_validation_create(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsDataValidationCreateReqBody, str, str, int) -> SpreadsheetsDataValidationCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsDataValidationCreateReqCall(self, body, request_opts=request_opts)

    def data_validation_delete(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsDataValidationDeleteReqBody, str, str, int) -> SpreadsheetsDataValidationDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsDataValidationDeleteReqCall(self, body, request_opts=request_opts)

    def data_validation_get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> SpreadsheetsDataValidationGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsDataValidationGetReqCall(self, request_opts=request_opts)

    def data_validation_update(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsDataValidationUpdateReqBody, str, str, int) -> SpreadsheetsDataValidationUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsDataValidationUpdateReqCall(self, body, request_opts=request_opts)

    def dimension_range_add(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsDimensionRangeAddReqBody, str, str, int) -> SpreadsheetsDimensionRangeAddReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsDimensionRangeAddReqCall(self, body, request_opts=request_opts)

    def dimension_range_delete(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsDimensionRangeDeleteReqBody, str, str, int) -> SpreadsheetsDimensionRangeDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsDimensionRangeDeleteReqCall(self, body, request_opts=request_opts)

    def dimension_range_update(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsDimensionRangeUpdateReqBody, str, str, int) -> SpreadsheetsDimensionRangeUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsDimensionRangeUpdateReqCall(self, body, request_opts=request_opts)

    def import_(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsImportReqBody, str, str, int) -> SpreadsheetsImportReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsImportReqCall(self, body, request_opts=request_opts)

    def import_result(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> SpreadsheetsImportResultReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsImportResultReqCall(self, request_opts=request_opts)

    def insert_dimension_range(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsInsertDimensionRangeReqBody, str, str, int) -> SpreadsheetsInsertDimensionRangeReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsInsertDimensionRangeReqCall(self, body, request_opts=request_opts)

    def merge_cells(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsMergeCellsReqBody, str, str, int) -> SpreadsheetsMergeCellsReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsMergeCellsReqCall(self, body, request_opts=request_opts)

    def metainfo(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> SpreadsheetsMetainfoReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsMetainfoReqCall(self, request_opts=request_opts)

    def protected_range_batch_create(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsProtectedRangeBatchCreateReqBody, str, str, int) -> SpreadsheetsProtectedRangeBatchCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsProtectedRangeBatchCreateReqCall(self, body, request_opts=request_opts)

    def protected_range_batch_delete(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsProtectedRangeBatchDeleteReqBody, str, str, int) -> SpreadsheetsProtectedRangeBatchDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsProtectedRangeBatchDeleteReqCall(self, body, request_opts=request_opts)

    def protected_range_batch_get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> SpreadsheetsProtectedRangeBatchGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsProtectedRangeBatchGetReqCall(self, request_opts=request_opts)

    def protected_range_batch_update(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsProtectedRangeBatchUpdateReqBody, str, str, int) -> SpreadsheetsProtectedRangeBatchUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsProtectedRangeBatchUpdateReqCall(self, body, request_opts=request_opts)

    def sheets_batch_update(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsSheetsBatchUpdateReqBody, str, str, int) -> SpreadsheetsSheetsBatchUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsSheetsBatchUpdateReqCall(self, body, request_opts=request_opts)

    def sheets_update_properties(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsSheetsUpdatePropertiesReqBody, str, str, int) -> SpreadsheetsSheetsUpdatePropertiesReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsSheetsUpdatePropertiesReqCall(self, body, request_opts=request_opts)

    def style_update(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsStyleUpdateReqBody, str, str, int) -> SpreadsheetsStyleUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsStyleUpdateReqCall(self, body, request_opts=request_opts)

    def styles_batch_update(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsStylesBatchUpdateReqBody, str, str, int) -> SpreadsheetsStylesBatchUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsStylesBatchUpdateReqCall(self, body, request_opts=request_opts)

    def unmerge_cells(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsUnmergeCellsReqBody, str, str, int) -> SpreadsheetsUnmergeCellsReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsUnmergeCellsReqCall(self, body, request_opts=request_opts)

    def values_append(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsValuesAppendReqBody, str, str, int) -> SpreadsheetsValuesAppendReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsValuesAppendReqCall(self, body, request_opts=request_opts)

    def values_batch_get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> SpreadsheetsValuesBatchGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsValuesBatchGetReqCall(self, request_opts=request_opts)

    def values_batch_update(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsValuesBatchUpdateReqBody, str, str, int) -> SpreadsheetsValuesBatchUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsValuesBatchUpdateReqCall(self, body, request_opts=request_opts)

    def values_get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> SpreadsheetsValuesGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsValuesGetReqCall(self, request_opts=request_opts)

    def values_image(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsValuesImageReqBody, str, str, int) -> SpreadsheetsValuesImageReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsValuesImageReqCall(self, body, request_opts=request_opts)

    def values_prepend(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsValuesPrependReqBody, str, str, int) -> SpreadsheetsValuesPrependReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsValuesPrependReqCall(self, body, request_opts=request_opts)

    def values_update(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (SpreadsheetsValuesUpdateReqBody, str, str, int) -> SpreadsheetsValuesUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SpreadsheetsValuesUpdateReqCall(self, body, request_opts=request_opts)



class SpreadsheetsConditionFormatsBatchCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsConditionFormatsBatchCreateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsConditionFormatsBatchCreateReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsConditionFormatsBatchCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/condition_formats/batch_create', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsConditionFormatsBatchCreateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsConditionFormatsBatchDeleteReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsConditionFormatsBatchDeleteReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsConditionFormatsBatchDeleteReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsConditionFormatsBatchDeleteResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/condition_formats/batch_delete', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsConditionFormatsBatchDeleteResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsConditionFormatsBatchGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (SpreadsheetsService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsConditionFormatsBatchGetReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def set_sheet_ids(self, sheet_ids):
        # type: (List[str]) -> SpreadsheetsConditionFormatsBatchGetReqCall
        self.query_params['sheet_ids'] = sheet_ids
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsConditionFormatsBatchGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/condition_formats', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=SpreadsheetsConditionFormatsBatchGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsConditionFormatsBatchUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsConditionFormatsBatchUpdateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsConditionFormatsBatchUpdateReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsConditionFormatsBatchUpdateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/condition_formats/batch_update', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsConditionFormatsBatchUpdateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsDataValidationCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsDataValidationCreateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsDataValidationCreateReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/dataValidation', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsDataValidationDeleteReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsDataValidationDeleteReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsDataValidationDeleteReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsDataValidationDeleteResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/dataValidation', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsDataValidationDeleteResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsDataValidationGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (SpreadsheetsService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsDataValidationGetReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsDataValidationGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/dataValidation', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=SpreadsheetsDataValidationGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsDataValidationUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsDataValidationUpdateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsDataValidationUpdateReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def set_sheetId(self, sheet_id):
        # type: (str) -> SpreadsheetsDataValidationUpdateReqCall
        self.path_params['sheetId'] = sheet_id
        return self

    def set_dataValidationId(self, data_validation_id):
        # type: (int) -> SpreadsheetsDataValidationUpdateReqCall
        self.path_params['dataValidationId'] = data_validation_id
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsDataValidationUpdateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/dataValidation/:sheetId/:dataValidationId', 'PUT', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsDataValidationUpdateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsDimensionRangeAddReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsDimensionRangeAddReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsDimensionRangeAddReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsDimensionRangeAddResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/dimension_range', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsDimensionRangeAddResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsDimensionRangeDeleteReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsDimensionRangeDeleteReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsDimensionRangeDeleteReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsDimensionRangeDeleteResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/dimension_range', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsDimensionRangeDeleteResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsDimensionRangeUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsDimensionRangeUpdateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsDimensionRangeUpdateReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/dimension_range', 'PUT', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsImportReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsImportReqBody, List[Any]) -> None

        self.service = service
        self.body = body

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[SpreadsheetsImportResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('sheets/v2/import', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsImportResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsImportResultReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (SpreadsheetsService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_ticket(self, ticket):
        # type: (str) -> SpreadsheetsImportResultReqCall
        self.query_params['ticket'] = ticket
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsImportResultResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('sheets/v2/import/result', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=SpreadsheetsImportResultResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsInsertDimensionRangeReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsInsertDimensionRangeReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsInsertDimensionRangeReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/insert_dimension_range', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsMergeCellsReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsMergeCellsReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsMergeCellsReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsMergeCellsResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/merge_cells', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsMergeCellsResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsMetainfoReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (SpreadsheetsService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsMetainfoReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def set_extFields(self, ext_fields):
        # type: (str) -> SpreadsheetsMetainfoReqCall
        self.query_params['extFields'] = ext_fields
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsMetainfoResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/metainfo', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=SpreadsheetsMetainfoResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsProtectedRangeBatchCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsProtectedRangeBatchCreateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsProtectedRangeBatchCreateReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsProtectedRangeBatchCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/protected_dimension', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsProtectedRangeBatchCreateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsProtectedRangeBatchDeleteReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsProtectedRangeBatchDeleteReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsProtectedRangeBatchDeleteReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsProtectedRangeBatchDeleteResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/protected_range_batch_del', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsProtectedRangeBatchDeleteResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsProtectedRangeBatchGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (SpreadsheetsService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsProtectedRangeBatchGetReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def set_protectIds(self, protect_ids):
        # type: (List[str]) -> SpreadsheetsProtectedRangeBatchGetReqCall
        self.query_params['protectIds'] = protect_ids
        return self

    def set_memberType(self, member_type):
        # type: (str) -> SpreadsheetsProtectedRangeBatchGetReqCall
        self.query_params['memberType'] = member_type
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsProtectedRangeBatchGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/protected_range_batch_get', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=SpreadsheetsProtectedRangeBatchGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsProtectedRangeBatchUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsProtectedRangeBatchUpdateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsProtectedRangeBatchUpdateReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsProtectedRangeBatchUpdateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/protected_range_batch_update', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsProtectedRangeBatchUpdateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsSheetsBatchUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsSheetsBatchUpdateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsSheetsBatchUpdateReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsSheetsBatchUpdateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/sheets_batch_update', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsSheetsBatchUpdateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsSheetsUpdatePropertiesReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsSheetsUpdatePropertiesReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsSheetsUpdatePropertiesReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsSheetsUpdatePropertiesResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/properties', 'PUT', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsSheetsUpdatePropertiesResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsStyleUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsStyleUpdateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsStyleUpdateReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsStyleUpdateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/style', 'PUT', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsStyleUpdateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsStylesBatchUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsStylesBatchUpdateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsStylesBatchUpdateReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsStylesBatchUpdateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/styles_batch_update', 'PUT', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsStylesBatchUpdateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsUnmergeCellsReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsUnmergeCellsReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsUnmergeCellsReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsUnmergeCellsResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/unmerge_cells', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsUnmergeCellsResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsValuesAppendReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsValuesAppendReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsValuesAppendReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def set_insertDataOption(self, insert_data_option):
        # type: (str) -> SpreadsheetsValuesAppendReqCall
        self.query_params['insertDataOption'] = insert_data_option
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsValuesAppendResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/values_append', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsValuesAppendResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsValuesBatchGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (SpreadsheetsService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsValuesBatchGetReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def set_ranges(self, ranges):
        # type: (List[str]) -> SpreadsheetsValuesBatchGetReqCall
        self.query_params['ranges'] = ranges
        return self

    def set_valueRenderOption(self, value_render_option):
        # type: (str) -> SpreadsheetsValuesBatchGetReqCall
        self.query_params['valueRenderOption'] = value_render_option
        return self

    def set_dateTimeRenderOption(self, date_time_render_option):
        # type: (str) -> SpreadsheetsValuesBatchGetReqCall
        self.query_params['dateTimeRenderOption'] = date_time_render_option
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsValuesBatchGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/values_batch_get', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=SpreadsheetsValuesBatchGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsValuesBatchUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsValuesBatchUpdateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsValuesBatchUpdateReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsValuesBatchUpdateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/values_batch_update', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsValuesBatchUpdateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsValuesGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (SpreadsheetsService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (SpreadsheetToken) -> SpreadsheetsValuesGetReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def set_range(self, range):
        # type: (SpreadsheetToken) -> SpreadsheetsValuesGetReqCall
        self.path_params['range'] = range
        return self

    def set_valueRenderOption(self, value_render_option):
        # type: (str) -> SpreadsheetsValuesGetReqCall
        self.query_params['valueRenderOption'] = value_render_option
        return self

    def set_dateTimeRenderOption(self, date_time_render_option):
        # type: (str) -> SpreadsheetsValuesGetReqCall
        self.query_params['dateTimeRenderOption'] = date_time_render_option
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsValuesGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/values/:range', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=SpreadsheetsValuesGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsValuesImageReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsValuesImageReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsValuesImageReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsValuesImageResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/values_image', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsValuesImageResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsValuesPrependReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsValuesPrependReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsValuesPrependReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsValuesPrependResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/values_prepend', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsValuesPrependResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SpreadsheetsValuesUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SpreadsheetsService, SpreadsheetsValuesUpdateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_spreadsheetToken(self, spreadsheet_token):
        # type: (str) -> SpreadsheetsValuesUpdateReqCall
        self.path_params['spreadsheetToken'] = spreadsheet_token
        return self

    def do(self):
        # type: () -> Response[SpreadsheetsValuesUpdateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('sheets/v2/spreadsheets/:spreadsheetToken/values', 'PUT', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SpreadsheetsValuesUpdateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp




# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
import attr




@to_json_decorator
@attr.s
class Identity(object):
    identity_name = attr.ib(type=str, default=None, metadata={'json': 'identity_name'})
    identity_code = attr.ib(type=str, default=None, metadata={'json': 'identity_code'})
    mobile = attr.ib(type=str, default=None, metadata={'json': 'mobile'})



@to_json_decorator
@attr.s
class IdentityCreateReqBody(object):
    identity_name = attr.ib(type=str, default=None, metadata={'json': 'identity_name'})
    identity_code = attr.ib(type=str, default=None, metadata={'json': 'identity_code'})
    mobile = attr.ib(type=str, default=None, metadata={'json': 'mobile'})


@attr.s
class IdentityCreateResult(object):
    verify_uid = attr.ib(type=str, default=None, metadata={'json': 'verify_uid'})


# coding: utf-8

from __future__ import absolute_import, division, print_function, unicode_literals

from copy import deepcopy
from typing import List, TypeVar

import attr

T = TypeVar('T')

int_to_string_fields = "__int_to_string_fields__"


def to_json(data):
    if not hasattr(data, '__dict__'):
        return data
    d = {}
    od = data.__dict__
    attr_field = attr.fields(data.__class__)
    for att in getattr(data, '__attrs_attrs__', []):
        att_name = att.name
        if att_name == int_to_string_fields:
            continue
        json_name = getattr(attr_field, att.name).metadata.get('json') or att_name
        v = od.get(att.name, att.default)
        if hasattr(v, 'json'):
            d[json_name] = v.json()
        elif isinstance(v, list):
            d[json_name] = [to_json(i) for i in v]
        elif isinstance(v, dict):
            d[json_name] = {kk: to_json(vv) for kk, vv in v.items()}
        elif att_name in od.get(int_to_string_fields, []) and v is not None:
            d[json_name] = str(v)
        elif v is not None:
            d[json_name] = v
    return d


def to_json_decorator(cls):
    cls.json = to_json
    return cls


def make_datatype(t, kwargs):
    """make_datatype

    :type t: Type[T]
    :type kwargs: Any
    :rtype: T
    """
    if not kwargs:
        return
    kwargs = deepcopy(kwargs)

    if isinstance(kwargs, (int, str, bool)):
        return kwargs

    if t.__class__ == List.__class__ and isinstance(kwargs, list):
        return [make_datatype(t.__args__[0], i) for i in kwargs]

    if not hasattr(t, '__attrs_attrs__'):
        return kwargs

    d = {}
    attr_field = attr.fields(t)
    for att in getattr(t, '__attrs_attrs__', []):
        att_name = att.name
        json_name = getattr(attr_field, att.name).metadata.get('json') or att_name
        att_default = att.default

        att_value = kwargs.get(json_name)
        if att_value:
            d[att_name] = make_datatype(att.type, att_value)
            del kwargs[json_name]
        elif att_default is attr.NOTHING:
            d[att_name] = None

    return t(**d)


def int_convert_bool(v):
    return bool(v)


def to_lower_converter(s):
    if isinstance(s, str):
        return s.lower()
    return s



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
import attr




@to_json_decorator
@attr.s
class Doc(object):
    pass



@to_json_decorator
@attr.s
class DocBatchUpdateReqBody(object):
    doc_token = attr.ib(type=str, default=None, metadata={'json': 'docToken'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'Revision'})
    requests = attr.ib(type=List[str], default=None, metadata={'json': 'Requests'})




@attr.s
class DocContentResult(object):
    content = attr.ib(type=str, default=None, metadata={'json': 'content'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})


@to_json_decorator
@attr.s
class DocCreateReqBody(object):
    folder_token = attr.ib(type=str, default=None, metadata={'json': 'FolderToken'})
    content = attr.ib(type=str, default=None, metadata={'json': 'Content'})


@attr.s
class DocCreateResult(object):
    obj_token = attr.ib(type=str, default=None, metadata={'json': 'objToken'})
    url = attr.ib(type=str, default=None, metadata={'json': 'url'})



@attr.s
class DocMetaResult(object):
    create_date = attr.ib(type=str, default=None, metadata={'json': 'create_date'})
    create_time = attr.ib(type=int, default=None, metadata={'json': 'create_time'})
    creator = attr.ib(type=str, default=None, metadata={'json': 'creator'})
    create_user_name = attr.ib(type=str, default=None, metadata={'json': 'create_user_name'})
    delete_flag = attr.ib(type=int, default=None, metadata={'json': 'delete_flag'})
    edit_time = attr.ib(type=int, default=None, metadata={'json': 'edit_time'})
    edit_user_name = attr.ib(type=str, default=None, metadata={'json': 'edit_user_name'})
    is_external = attr.ib(type=bool, default=None, metadata={'json': 'is_external'})
    is_pined = attr.ib(type=bool, default=None, metadata={'json': 'is_pined'})
    is_stared = attr.ib(type=bool, default=None, metadata={'json': 'is_stared'})
    obj_type = attr.ib(type=str, default=None, metadata={'json': 'obj_type'})
    owner = attr.ib(type=str, default=None, metadata={'json': 'owner'})
    owner_user_name = attr.ib(type=str, default=None, metadata={'json': 'owner_user_name'})
    server_time = attr.ib(type=int, default=None, metadata={'json': 'server_time'})
    tenant_id = attr.ib(type=str, default=None, metadata={'json': 'tenant_id'})
    title = attr.ib(type=str, default=None, metadata={'json': 'title'})
    type = attr.ib(type=int, default=None, metadata={'json': 'type'})
    url = attr.ib(type=str, default=None, metadata={'json': 'url'})



@attr.s
class DocRawContentResult(object):
    content = attr.ib(type=str, default=None, metadata={'json': 'content'})


# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
import attr




@to_json_decorator
@attr.s
class Term(object):
    from_ = attr.ib(type=str, default=None, metadata={'json': 'from'})
    to = attr.ib(type=str, default=None, metadata={'json': 'to'})


@to_json_decorator
@attr.s
class Text(object):
    pass



@to_json_decorator
@attr.s
class TextTranslateReqBody(object):
    source_language = attr.ib(type=str, default=None, metadata={'json': 'source_language'})
    text = attr.ib(type=str, default=None, metadata={'json': 'text'})
    target_language = attr.ib(type=str, default=None, metadata={'json': 'target_language'})
    glossary = attr.ib(type=List[Term], default=None, metadata={'json': 'glossary'})


@attr.s
class TextTranslateResult(object):
    text = attr.ib(type=str, default=None, metadata={'json': 'text'})


@to_json_decorator
@attr.s
class TextDetectReqBody(object):
    text = attr.ib(type=str, default=None, metadata={'json': 'text'})


@attr.s
class TextDetectResult(object):
    language = attr.ib(type=str, default=None, metadata={'json': 'language'})